<?php

/**
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or (at
 *   your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful, but
 *   WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 *   @author          BlackBird Webprogrammierung
 *   @copyright       2016, BlackBird Webprogrammierung
 *   @link            http://blackcat-cms.org
 *   @license         http://www.gnu.org/licenses/gpl.html
 *   @category        CAT_Modules
 *   @package         DataTable
 *
 */

if (defined('CAT_PATH')) {
    if (defined('CAT_VERSION')) include(CAT_PATH.'/framework/class.secure.php');
} elseif (file_exists($_SERVER['DOCUMENT_ROOT'].'/framework/class.secure.php')) {
    include($_SERVER['DOCUMENT_ROOT'].'/framework/class.secure.php');
} else {
    $subs = explode('/', dirname($_SERVER['SCRIPT_NAME']));    $dir = $_SERVER['DOCUMENT_ROOT'];
    $inc = false;
    foreach ($subs as $sub) {
        if (empty($sub)) continue; $dir .= '/'.$sub;
        if (file_exists($dir.'/framework/class.secure.php')) {
            include($dir.'/framework/class.secure.php'); $inc = true;    break;
        }
    }
    if (!$inc) trigger_error(sprintf("[ <b>%s</b> ] Can't include class.secure.php!", $_SERVER['SCRIPT_NAME']), E_USER_ERROR);
}

global $tablename, $pg, $page_id, $section_id;

require __DIR__.'/info.php';

$tablename     = 'mod_'.$module_directory;
$pg            = CAT_Helper_Page::getInstance();

// ----- delete item(s) -----
function dt_delete()
{
    $items = CAT_Helper_Validate::sanitizePost('item[]');
}

// ----- get csv files in media dir -----
function dt_get_csvfiles()
{
    $user  = CAT_Users::getInstance();
    if($user->checkPermission('media','media_view',false) !== true)
        return false;

    $path  = CAT_Helper_Directory::sanitizePath(CAT_PATH.MEDIA_DIRECTORY);
    $files = CAT_Helper_Directory::getInstance()
           ->setSuffixFilter(array('csv'))
           ->findFiles('.*\.csv',$path,true);

    return $files;
}   // end function dt_get_csvfiles()

// ----- get data -----
function dt_get_data($id=NULL,$active_only=false)
{
    global $pg, $tablename, $section_id;
    $data  = array();

    $query = sprintf(
          'SELECT * FROM `:prefix:%s` AS `t1` '
        . 'WHERE `section_id`=? '
        . ( $id ? 'AND `id`="'.$id.'" ' : '' )
        . ( $active_only ? 'AND `active`="Y"' : '' ),
        $tablename.'_items'
    );

#echo "$query\n$section_id\n";

    $stmt  = $pg->db()->query($query,array($section_id));
    $items = $stmt->fetchAll();
#print_r($items);

    if(count($items))
    {
        $query = sprintf(
              'SELECT `id`, `field`, `value` FROM `:prefix:%s` AS `t1` '
            . 'WHERE `section_id`=? '
            . ( $id ? 'AND `id`="'.$id.'" ' : '' ),
            $tablename.'_data'
        );
        $stmt  = $pg->db()->query($query,array($section_id));
        $fields = $stmt->fetchAll();

        foreach($items as $item)
        {
            $data[$item['id']] = array('id'=>$item['id'],'active'=>$item['active']);
            foreach($fields as $field)
            {
                if($field['id'] == $item['id'])
                {
                    $data[$item['id']]['field'.$field['field']] = $field['value'];
                }
            }
        }
    }
#print_r($data);

    return $data;
}

// ----- get field titles -----
function dt_get_fields($settings)
{
    global $pg, $tablename, $section_id;
    $fieldtitles = $fields = $req = array();

    $stmt = $pg->db()->query(sprintf(
        'SELECT * FROM `:prefix:%s` WHERE `section_id`=?',
        $tablename.'_fieldnames'
    ),array($section_id));
    $data = $stmt->fetchAll();

    foreach(array_values($data) as $field)
    {
        $fieldtitles[$field['fieldno']] = $field['title'];
        if($field['req']=='Y') $req[$field['fieldno']] = 'Y';
    }

    $fieldno  = $settings['fieldno'];

    foreach(range(1,$fieldno) as $no)
    {
        $fields[] = 'field'.$no;
    }

    return array($fields,$fieldtitles,$req);
}

function dt_import_csv($section_id,$settings)
{
    global $tablename;

    $val       = CAT_Helper_Validate::getInstance();
    $mediafile = $val->sanitizePost('csvfile');
    $delim     = $val->sanitizePost('delim');
    $maxrows   = $val->sanitizePost('import_lines');
    $maxcols   = $val->sanitizePost('import_columns');

    if(!$delim)
        $delim = ',';
    if($val->sanitizePost('lines')=='all')
        $maxrows = false;
    elseif($val->sanitizePost('firstline')=='y')
        $maxrows++;
    if($val->sanitizePost('columns')=='all')
        $maxcols = false;

    $line_endings = ini_get('auto_detect_line_endings');
    //ini_set('auto_detect_line_endings',TRUE);
    $fp = fopen(CAT_Helper_Directory::sanitizePath(CAT_PATH.MEDIA_DIRECTORY.'/'.$mediafile), "r");
    if(!$fp)
        CAT_Object::printFatalError('Unable to open the media file!');
    $lines = array();
    $count = 0;

    while(!feof($fp))
    {
        $lines[] = fgetcsv($fp,0,$delim,"\"");
        $count++;
        if($maxrows && $count==$maxrows)
            break;
    }
    fclose($fp);

    if(count($lines))
    {
        $cols_in_csv = count($lines[0]);

        if( !$maxcols && $cols_in_csv > $settings['fieldno'] )
        {
            return $val->lang()->translate(
                'The CSV file has more columns ({{columns}}) than you have configured for your table ({{fieldno}})!<br />You can configure to import just the first {{fieldno}} columns below.',
                array('columns'=>$cols_in_csv,'fieldno'=>$settings['fieldno'])
            );
        }

        if($val->sanitizePost('firstline')==='y')
        {
            array_shift($lines);
        }
        foreach($lines as $index => $line)
        {
            $val->db()->query(sprintf(
                'INSERT INTO `:prefix:%s` (`section_id`, `active`) VALUES (?,?)',
                $tablename.'_items'
            ), array($section_id, 'y'));
            $item_id = $val->db()->lastinsertid();
            if($maxcols)
            {
                $line = array_splice($line,0,$maxcols);
            }
            $fieldno = 1;
            foreach(array_values($line) as $item)
            {
                $val->db()->query(sprintf(
                    'INSERT INTO `:prefix:%s` (`section_id`, `id`, `field`, `value`) VALUES (?,?,?,?)',
                    $tablename.'_data'
                ), array($section_id, $item_id, $fieldno, $item));
                $fieldno++;
            }
        }
    }

    return true;
}

// ----- get settings -----
function dt_get_settings()
{
    global $pg, $tablename, $section_id;
    $stmt = $pg->db()->query(sprintf(
        'SELECT * FROM `:prefix:%s` WHERE `section_id`=?',
        $tablename.'_settings'
    ),array($section_id));
    $settings = $stmt->fetch();
    return $settings;
}

// print error message
function dt_error($msg)
{
    global $pg;
    echo "<div class=\"warning mod_datatable_warning\">",
         $pg->lang()->translate($msg),
         "</div><br /><br />";
}

function dt_success($msg)
{
    global $pg;
    echo "<div class=\"success mod_datatable_success\">",
         $pg->lang()->translate($msg),
         "</div>";
}