<?php

/**
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or (at
 *   your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful, but
 *   WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 *   @author          BlackBird Webprogrammierung
 *   @copyright       2016, BlackBird Webprogrammierung
 *   @link            http://blackcat-cms.org
 *   @license         http://www.gnu.org/licenses/gpl.html
 *   @category        CAT_Modules
 *   @package         DataTable
 *
 */

if (defined('CAT_PATH')) {
    if (defined('CAT_VERSION')) include(CAT_PATH.'/framework/class.secure.php');
} elseif (file_exists($_SERVER['DOCUMENT_ROOT'].'/framework/class.secure.php')) {
    include($_SERVER['DOCUMENT_ROOT'].'/framework/class.secure.php');
} else {
    $subs = explode('/', dirname($_SERVER['SCRIPT_NAME']));    $dir = $_SERVER['DOCUMENT_ROOT'];
    $inc = false;
    foreach ($subs as $sub) {
        if (empty($sub)) continue; $dir .= '/'.$sub;
        if (file_exists($dir.'/framework/class.secure.php')) {
            include($dir.'/framework/class.secure.php'); $inc = true;    break;
        }
    }
    if (!$inc) trigger_error(sprintf("[ <b>%s</b> ] Can't include class.secure.php!", $_SERVER['SCRIPT_NAME']), E_USER_ERROR);
}

require_once __DIR__.'/functions.php';

$parser->setPath(__DIR__.'/templates/default');

$settings      = dt_get_settings();
$items         = dt_get_data();
$settings_open = false;
list($fields,$fieldtitles,$req) = dt_get_fields($settings);

if(
       CAT_Helper_Validate::sanitizeGet('csvimport')
    || CAT_Helper_Validate::sanitizePost('datatable_import_save')
) {
    if(
           CAT_Helper_Validate::sanitizePost('csvfile')
        && file_exists(CAT_Helper_Directory::sanitizePath(CAT_PATH.MEDIA_DIRECTORY.CAT_Helper_Validate::sanitizePost('csvfile')))
    ) {
        if(true!==($result=dt_import_csv($section_id,$settings)))
        {
            dt_error($result);
        }
    }
    $parser->output(
        'csvimport.tpl',
        array(
            'section_id'       => $section_id,
            'page_id'          => $page_id,
            'settings'         => $settings,
            'module_directory' => $module_directory,
            'csvfiles'         => dt_get_csvfiles(),
            'form'             => (isset($_POST['datatable_import_save']) ? $_POST : array('firstline'=>'y','columns_all'=>'on')),
        )
    );
}
else
{
    // ----- save settings? -----
    if(CAT_Helper_Validate::sanitizePost('datatable_set_save'))
    {
        $fields   = array();
        $params   = array();
        $error    = false;

        $fieldno  = CAT_Helper_Validate::sanitizePost('fieldno');
        $title    = CAT_Helper_Validate::sanitizePost('subject','scalar');

        if($fieldno) // check the number of fields
        {
            if($fieldno < $settings['fieldno'] && count($items))
            {
                dt_error('It is not possible to decrease the number of fields if the table already has data!');
                $error = true;
            }
            else
            {
                $pg->db()->query(
                    'UPDATE `:prefix:'.$tablename.'_settings` SET `fieldno`=? WHERE `section_id`=?',
                    array($fieldno,$section_id)
                );
                if($pg->db()->isError())
                {
                    dt_error($pg->db()->getError());
                    $error = true;
                }
                else
                {
                    if($fieldno < $settings['fieldno'])
                    {
                        $pg->db()->query(
                            'DELETE FROM `:prefix:'.$tablename.'_fieldnames` WHERE `section_id`=? AND `fieldno`>?',
                            array($section_id,$fieldno)
                        );
                    }
                }
            }
        }

        // check if subject changed
        if(!$error)
        {
            if(strcasecmp($title,$settings['subject']))
            {
                $pg->db()->query(
                    'UPDATE `:prefix:'.$tablename.'_settings` SET `subject`=? WHERE `section_id`=?',
                    array($title,$section_id)
                );
                if($pg->db()->isError())
                {
                    dt_error($pg->db()->getError());
                    $error = true;
                }
            }
        }

        // check for hide empty setting
        if(!$error)
        {
            if(strcasecmp(CAT_Helper_Validate::sanitizePost('hide_empty'),$settings['hide_empty']))
            {
                $pg->db()->query(
                    'UPDATE `:prefix:'.$tablename.'_settings` SET `hide_empty`=? WHERE `section_id`=?',
                    array(CAT_Helper_Validate::sanitizePost('hide_empty'),$section_id)
                );
                if($pg->db()->isError())
                {
                    dt_error($pg->db()->getError());
                    $error = true;
                }
            }
        }

        // check the field titles
        if(!$error)
        {
            foreach(range(1,$settings['fieldno']) as $i)
            {
                #set_field3name
                if(($value=CAT_Helper_Validate::sanitizePost('set_field'.$i.'name'))!='')
                {
                    $is_required = CAT_Helper_Validate::sanitizePost('set_field'.$i.'name_req');
                    if(!$is_required) $is_required = 'N';
                    $pg->db()->query(
                        'REPLACE INTO `:prefix:'.$tablename.'_fieldnames` VALUES ( ?, ?, ?, ? )',
                        array($section_id, $i, $value, $is_required)
                    );
                    if($pg->db()->isError())
                    {
                        dt_error($pg->db()->getError());
                        $error = true;
                    }
                }
            }
        }

        // check result
        if(!$error)
        {
            dt_success('Options saved.');
            // reload settings
            $settings = dt_get_settings();
        }
        else
        {
            $settings_open = true;
            // todo: $settings mit $_POST kombinieren
        }

    }   // ----- end save settings -----

    // ----- add / edit item? -----
    if(CAT_Helper_Validate::sanitizePost('datatable_item_add'))
    {
        $formdata = array();
        $params   = array();
        $error    = false;

        // check for required fields
        // should never happen in modern browsers
        foreach(array_keys($req) as $i)
        {
            if(!CAT_Helper_Validate::sanitizePost('field'.$i))
            {
                dt_error(CAT_Helper_Validate::getInstance()->lang()->translate(
                    'Item cannot be saved. Missing data for field [{{field}}].',
                    array('field'=>$fieldtitles[$i])
                ));
                $error = true;
            }
        }

        if(!$error)
        {
            foreach(range(1,$settings['fieldno']) as $i)
            {
                if(""!=($value=CAT_Helper_Validate::sanitizePost('field'.$i)))
                {
                    $formdata[] = '`field'.$i.'`';
                    $params[]   = $value;
                }
            }
            if(!count($formdata))
            {
                dt_error('Item cannot be saved. No data.');
                $error = true;
            }
            else
            {

                if(CAT_Helper_Validate::sanitizePost('item_id')=='-1')
                {
                    $pg->db()->query(sprintf(
                        'INSERT INTO `:prefix:%s_items` ( `section_id`, `active` ) VALUES (?,?)',
                        $tablename
                    ),array($section_id,CAT_Helper_Validate::sanitizePost('active')));
                    $item_id = $pg->db()->lastinsertid();

                    $query    = 'INSERT INTO `:prefix:'.$tablename.'_data` ( `section_id`, `id`, `field`, `value` ) '
                              . 'VALUES ( ?, ?, ?, ? )'
                              ;

                    foreach($formdata as $i => $field)
                    {
                        $pg->db()->query($query,array($section_id,$item_id,($i+1),$params[$i]));
                        if($pg->db()->isError()) {
                            dt_error(
                                $pg->lang()->translate("Error saving new item") . ": " .
                                $pg->db()->getError()
                            );
                        }
                    }
                }
                else
                {
                    $pg->db()->query(sprintf(
                        'UPDATE `:prefix:%s_items` SET `active`=? WHERE `id`=?',
                        $tablename
                    ),array(CAT_Helper_Validate::sanitizePost('active'),CAT_Helper_Validate::sanitizePost('item_id')));

                    foreach($formdata as $i => $field)
                    {
                        $pg->db()->query(sprintf(
                            'UPDATE `:prefix:%s_data` SET `value`=? WHERE `field`=? AND `id`=?',
                            $tablename
                        ),array($params[$i],($i+1),CAT_Helper_Validate::sanitizePost('item_id')));
                    }
                }

                dt_success('Item saved.');
                // reload items
                $items = dt_get_data();

            }
        }
    }   // ----- end add item -----

    $parser->output(
        'modify.tpl',
        array(
            'section_id'       => $section_id,
            'page_id'          => $page_id,
            'settings'         => $settings,
            'fields'           => $fields,
            'titles'           => $fieldtitles,
            'items'            => $items,
            'settings_open'    => $settings_open,
            'module_directory' => $module_directory,
            'req'              => $req,
            'csvfiles'         => dt_get_csvfiles(),
            'addandopen'       => (CAT_Helper_Validate::sanitizePost('datatable_item_add')==2?true:false),
        )
    );

}   // end if(CAT_Helper_Validate::sanitizeGet('csvimport'))